/*********************************************
/ BotActuators.cpp
/
/ This file contains all of the actuator definitions
/ for the bot. The actuators are all defined as member
/ functions of the CBot class. The speed constant
/ table for translation and strafing is also defined
/ in this file.
*********************************************/

#include "extdll.h" 
#include "util.h"
#include "client.h"
#include "cbase.h"
#include "player.h"
#include "items.h"
#include "weapons.h"
#include "soundent.h"
#include "gamerules.h"
#include "BotMessenger.h"
#include "ExternalBotDebug.h"
#include "pm_debug.h"
#include "effects.h"
#include "time.h"
#include "BotCommunication.h"

#include "bot.h"

// Speed constant table (for translation and strafing)
float bot_move_speeds[] = { 0.0,
							50.0,
							300.0 };

/*********************************************
/ DO TRANSLATION
/ Causes the bot to translate (move forward)
/ at the desired speed. Speeds are the same as
/ for strafing (see the speed constant table)
*********************************************/
void CBot::do_translation(int speed)
{
	// Lookup the appropriate speed for the speed constant
	f_move_speed = bot_move_speeds[abs(speed)];

	// Make player go backward if the speed is negative
	if (speed < 0)
	{
		f_move_speed *= -1;
		pev->button |= IN_BACK;
	}else{
		pev->button |= IN_FORWARD;
	}
}

/*********************************************
/ DO ROTATE
/ Causes the bot to rotate at the desired speed.
/ Note that these speeds are not the same as for
/ translation or strafing. Positive speeds turn
/ left and negative speeds turn right. Speed is
/ in 1/10 degrees per second.
*********************************************/
void CBot::do_rotation(int speed)
{
	// Don't make all of these calculations if there will be no change
	if (speed == 0)
		return;
	
	float current;
	float offset;

	if(newTurnSignsP){
		// Calculate the offset by which to rotate
		offset = -speed * .1 * (gpGlobals->time - lastThinkTime);
		
		// Make the rotation
		current = pev->v_angle.y;
		current += offset;
		
		// Check for wrap around of angle
		if (current < 0)
			current += 360;
		if (current >= 360)
			current -= 360;

	}else{
		// Calculate the offset by which to rotate
		float offset = speed * .1 * (gpGlobals->time - lastThinkTime);
		
		// Make the rotation
		current = pev->v_angle.y;
		current += offset;
		
		// Check for wrap around of angle
		if (current < 0)
			current += 360;
		if (current >= 360)
			current -= 360;
	}

	// Set the final angles (move the body with the head)
	pev->v_angle.y = current;
	pev->angles.y = pev->v_angle.y;
	pev->angles.z = 0;
}

/*********************************************
/ DO STRAFE
/ Causes the bot to strafe (move side-to-side)
/ at the desired speed. Speeds are the same as
/ for translation (see the speed constant table)
*********************************************/
void CBot::do_strafe(int speed)
{
	// Lookup the appropriate speed for the speed constant
	f_strafe_speed = bot_move_speeds[abs(speed)];

	// Make player go backward if the speed is negative
	if (speed < 0) f_strafe_speed *= -1;
}

/*********************************************
/ DO PITCH
/ Causes the bot to change pitch (up-down look angle)
/ at the desired speed. Note that these speeds are not
/ the same as for translation, strafing, or rotation.
/ Negative speeds pitch down (towards the ground) and
/ positive speeds pitch up (towards the sky). Speed is
/ in 1/10 degrees per second.
*********************************************/
void CBot::do_pitch(int speed)
{
	// Don't make all of these calculations if there will be no change
	if (speed == 0)
		return;

	// Calculate the offset by which to change pitch
	float offset = speed * .1 * (gpGlobals->time - lastThinkTime);

	// Make the rotation
	float current = -pev->v_angle.x;
	current += offset;

	// Check for wrap around of angle
	if (current < -180) 
		current = -180;
	else if	(current > 180)
		current = 180;

	// set the view and body angles to point the gun correctly
	pev->v_angle.x = current;
	pev->angles.x = pev->v_angle.x / 3;		// it is unknown why this must be divided by 3
	pev->v_angle.x = -pev->v_angle.x;		// adjust pitch to point gun (engine bug fix)
}

/*********************************************
/ DO SHOOT
/ Causes the bot to shoot (depress the attack button).
*********************************************/
void CBot::do_shoot(bool do_it)
{
	if (do_it)
		pev->button |= IN_ATTACK;
}

/*********************************************
/ DO JUMP
/ Causes the bot to jump (depress the jump button).
*********************************************/
void CBot::do_jump(bool do_it)
{
	if (do_it){
		pev->button |= IN_JUMP;
	}else{
		pev->button &= ~IN_JUMP;
	}	
}

/*********************************************
/ DO RELOAD
/ Causes the bot to reload (depress the reload button).
*********************************************/
void CBot::do_reload(bool do_it)
{
	if (do_it)
		pev->button |= IN_RELOAD;
}

/*********************************************
/ DO DUCK
/ Causes the bot to duck (depress the duck button).
*********************************************/
void CBot::do_duck(bool do_it)
{
	if (do_it)
		pev->button |= IN_DUCK;
}

/*********************************************
/ DO SECONDARY FIRE
/ Causes the bot to shoot (depress the attack button).
*********************************************/
void CBot::do_secondary(bool do_it)
{
	if (do_it)
		pev->button |= IN_ATTACK2;
}

/*********************************************
/ DO WEAPON SWITCH
/ Causes the bot to shoot (depress the attack button).
*********************************************/
void CBot::do_weapon(int do_it)
{

	switch(do_it){

	case WEAPON_NONE:
		break;
	case WEAPON_CROWBAR:
		SelectItem("weapon_crowbar");
		break;
	case WEAPON_GLOCK:
		SelectItem("weapon_9mmhandgun");
		break;
	case WEAPON_SHOTGUN:
		SelectItem("weapon_shotgun");	
		break;
	case WEAPON_GAUSS:
		SelectItem("weapon_gauss");
		break;
	case WEAPON_EGON:
		SelectItem("weapon_egon");
		break;
	case WEAPON_MP5:
		SelectItem("weapon_9mmAR");
		break;
	case WEAPON_RPG:
		SelectItem("weapon_rpg");
		break;
	case WEAPON_PYTHON:
		SelectItem("weapon_357");
		break;
	case WEAPON_HORNETGUN:
		SelectItem("weapon_hornetgun");
		break;
	case WEAPON_CROSSBOW:
		SelectItem("weapon_crossbow");
		break;
	case WEAPON_SNARK:
		SelectItem("weapon_snark");
		break;
	case WEAPON_HANDGRENADE:
		SelectItem("weapon_handgrenade");
		break;
	case WEAPON_TRIPMINE:
		SelectItem("weapon_tripmine");
		break;
	case WEAPON_SATCHEL:
		SelectItem("weapon_satchel");
		break;
	default:
		return;
	}

	return;
}

/*********************************************
/ DO USE
/ Causes the bot to depress the use button.
*********************************************/
void CBot::do_use(bool do_it)
{
	if (do_it)
		pev->button |= IN_USE;
}

/*********************************************
/ ADD WAYPOINT
/ Adds a waypoint in the bot's current position.
*********************************************/
void CBot::add_waypoint()
{
	WAYPOINT wp;
	wp.origin = pev->origin;
	wp.type = WAYPOINT_CHOKEPOINT;
	AddWaypoint(wp);
}

/*********************************************
/ ADD INFOPOINT
/ Adds a infopoint in the bot's current position.
*********************************************/
void CBot::add_infopoint(int type,int lifespanMs)
{
	if(type <= 0 || type > 100)
		return;

	WAYPOINT wp;
	wp.origin = pev->origin;
	wp.type = type;
	
	AddInfopoint(wp, gpGlobals->time, lifespanMs);
}

/*********************************************
/ BOT COMMUNICATION
/ these actuators are for setting the bots 
/ values there are sensors that other bots can use
/ to read back these values for other bots
*********************************************/
/*********************************************
/ COMMUNICATION SET INT
*********************************************/
void CBot::comms_set_int(int box, int number){
	hivemind_set(type_integer, box , playerNum, number);
}

/*********************************************
/ COMMUNICATION SET FLOAT
*********************************************/
void CBot::comms_set_float(int box, float number){
	hivemind_set(type_float, box, playerNum, number);
}

/*********************************************
/ COMMUNICATION SET BOOL
*********************************************/
void CBot::comms_set_bool(int box, bool value){
	hivemind_set(type_bool, box, playerNum, value);
}


