/*********************************************
/ BotSensors.cpp
/
/ This file contains all of the sensor definitions
/ for the bot. The sensors are all defined as member
/ functions of the CBot class.
*********************************************/

#include "extdll.h" 
#include "util.h"
#include "client.h"
#include "cbase.h"
#include "weapons.h"

#include "soundent.h"

#include "gamerules.h"

#include "assert.h"
#include "bot.h"

#include "BotSurroundings.h"
#include "BotCommunication.h"

extern CSoundEnt *pSoundEnt;
extern WaypointBeamMarker waypointBeams[MAX_NUM_WAYPOINTS * MAX_CLIENTS];
extern WaypointBeamMarker InfopointBeams[MAX_NUM_WAYPOINTS * MAX_CLIENTS];
/*********************************************
/ GET HEALTH
/ Returns the health of the player as an integer.
*********************************************/
int CBot::get_health()
{
	// m_iClientHealth is inherited from CBasePlayer
	return m_iClientHealth;
}

/*********************************************
/ GET YAW
/ Returns the bot's absolute yaw (left-to-right)
/ with respect to the world.
*********************************************/
float CBot::get_yaw()
{
	return pev->v_angle.y;
}

/*********************************************
/ GET PITCH
/ Returns the bot's absolute pitch (up and down)
/ with respect to the world.
*********************************************/
float CBot::get_pitch()
{
	if(newTurnSignsP){
		return -pev->v_angle.x;
	}else{
		return pev->v_angle.x;
	}

}

/*********************************************
/ GET SWEEP INFO
/ Calculates the distance the closest object hit 
/ in a sweep of tracelines (within the given sweep
/ range at the given intervals). Also finds what
/ kind of object it is touching (the class of the
/ entity -- e.g. "player"). A VisionResult is
/ returned instead of an integer distance to give
/ it the ability to determine what the obstacle is.
*********************************************/
VisionResult CBot::get_sweep_info(float lookAngle, int checkDistance, int sweep, int interval) {
	VisionResult min = get_vision_info(lookAngle - sweep, checkDistance, NULL);
	VisionResult thisturn;
	for (int i = lookAngle - sweep; i < lookAngle + sweep; i+=interval){
		thisturn = get_vision_info(i, checkDistance, NULL);
		if (thisturn.vdistance < min.vdistance){
			min = thisturn;
		}
	}
	return min;
}

/*********************************************
/ GET LEFT DISTANCE
/ Calculates the distance (rounded to an int)
/ to the closest object hit in a series of tracelines
/ in the left-hand portion of the bot's field of view
*********************************************/
int CBot::get_left_distance(int checkDistance) {
	if (m_bDebugOn)
	{
		if (!m_bUpdatingLeftBeam)
			m_bUpdatingLeftBeam = TRUE;
		get_vision_info(FIELD_OF_VIEW / 2, checkDistance, m_pLeftBeam).vdistance;
	}
	return (int)get_sweep_info(40, checkDistance, 20, 8).vdistance;
}

/*********************************************
/ GET RIGHT DISTANCE
/ Calculates the distance (rounded to an int)
/ to the closest object hit in a series of tracelines
/ in the right-hand portion of the bot's field of view
*********************************************/
int CBot::get_right_distance(int checkDistance) {
	if (m_bDebugOn)
	{
		if (!m_bUpdatingRightBeam)
			m_bUpdatingRightBeam = TRUE;
		get_vision_info(- FIELD_OF_VIEW / 2, checkDistance, m_pRightBeam).vdistance;
	}
	return (int)get_sweep_info(-40, checkDistance, 20, 8).vdistance;
}

/*********************************************
/ GET CENTER DISTANCE
/ Calculates the distance (rounded to an int)
/ to the closest object hit in a series of tracelines
/ in the center portion of the bot's field of view
*********************************************/
int CBot::get_center_distance(int checkDistance) {
	if (m_bDebugOn)
	{
		if (!m_bUpdatingCenterBeam)
			m_bUpdatingCenterBeam = TRUE;
		get_vision_info(0, checkDistance, m_pCenterBeam).vdistance;
	}
	return (int)get_sweep_info(0, checkDistance, 20, 8).vdistance;
}

/*********************************************
/ IS ENTITY VISIBLE
/ Determines whether the entity is visible to the
/ bot by checking:
/   1. is the entity's location within the field of view?
/   2. can a line be traced to the entity?
*********************************************/
bool CBot::IsEntityVisible(CBaseEntity * pEntity)
{
	if(!pEntity)
		return FALSE;
	if(!pEntity->pev)
		return FALSE;
// Check if the player is within the field of view
	if (fabs(YawToEntity(pEntity)) <= FIELD_OF_VIEW / 2)
	{
		Vector vecLookerOrigin = pev->origin + pev->view_ofs;//look through the caller's 'eyes'
		
		// Make sure the target has a valid pev
		if (!pEntity->pev)
			return FALSE;

		Vector vecTargetOrigin = pEntity->pev->origin;

		TraceResult tr;
		UTIL_TraceLine(vecLookerOrigin, vecTargetOrigin, ignore_monsters, ignore_glass, ENT(pev)/*pentIgnore*/, &tr);

		return (tr.flFraction == 1.0);
	}
	return false;
}

/*********************************************
/ IS ENTITY VISIBLE
/ Determines whether the entity is visible to the
/ bot by checking:
/   1. is the entity's location within the field of view?
/   2. can a line be traced to the entity?
*********************************************/
bool CBot::IsLocationVisible(Vector location)
{

	// Check if the location is within the field of view
	if (fabs(YawToLocation(location)) <= FIELD_OF_VIEW / 2)
	{
		Vector vecLookerOrigin = pev->origin + pev->view_ofs;//look through the caller's 'eyes'
		
		Vector vecTargetOrigin = location;

		TraceResult tr;
		UTIL_TraceLine(vecLookerOrigin, vecTargetOrigin, ignore_monsters, ignore_glass, ENT(pev)/*pentIgnore*/, &tr);
		
		return (tr.flFraction == 1.0);
	}
	return false;
}

/*********************************************
/ IS PLAYER VISIBLE
/ Determines whether the player is visible to the
/ bot by checking:
/   1. is the player's location within the field of view?
/   2. can a line be traced to the player?
*********************************************/
bool CBot::is_player_visible(int lookPlayerNum)
{
	// Check if the player is within the field of view
	if (fabs(yaw_to_player(lookPlayerNum)) <= FIELD_OF_VIEW / 2)
	{
		CBaseEntity * pPlayerToFind = UTIL_PlayerByIndex(lookPlayerNum);
		if (pPlayerToFind == NULL)
			return FALSE;

		// Skip this player if not alive (i.e. dead or dying)
		if (pPlayerToFind->pev->deadflag != DEAD_NO)
			return FALSE;
	
		// Skip players that are in botcam mode...
		if (pPlayerToFind->pev->effects & EF_NODRAW)
			return FALSE;

		if (FVisible (pPlayerToFind) && pPlayerToFind->IsAlive())
			return true;
	}
	return false;
}

/*********************************************
/ CALCULATE SENSOR ERROR
/ Calculates the sensor error depending on the
/ bot's skill level. Returns a float that is
/ a percentage error to be introduced into
/ the sensor results (e.g. 20% == 0.2).
*********************************************/
float CBot::CalculateSensorError(int playerNum)
{
	float undecayTime = 3.0;
	float initialError = 3.0;
	float decayAmount = 0.1;
	float decayPeriod = 0.2;

	// No error if this is a teammate
	CBasePlayer * pPlayer = (CBasePlayer *)UTIL_PlayerByIndex(playerNum);
	if (UTIL_TeamsMatch(TeamID(), pPlayer->TeamID()))
	{
		return 0;
	}

	float interval = gpGlobals->time - last_player_look_time[playerNum];

	// Is it time to undecay (hasn't looked at this player for a while)?
	if (interval >= undecayTime || last_player_look_time[playerNum] < 0.0)
	{
		last_player_look_time[playerNum] = gpGlobals->time;
		player_look_error[playerNum] = initialError;
		interval = 0.0001;
	}

	// Decay the error if necessary
	int numDecays = (int)(roundDecimal(interval, 1) / decayPeriod);
	if (numDecays > 0)
	{
		for (int i = 0; i < numDecays; i++)
			player_look_error[playerNum] -= decayAmount;
		last_player_look_time[playerNum] = gpGlobals->time;
		if (player_look_error[playerNum] < 0)
			player_look_error[playerNum] = 0;
	}

	return player_look_error[playerNum];
}

/*********************************************
/ YAW TO PLAYER
/ Calculates the yaw (angle) between where the bot
/ is looking and a player of client index lookPlayerNum.
/
/ Return values are -180 <= yaw <= 180. If the player
/ is to the right of center, the return value will be
/ negative. To the left, the return will be positive.
*********************************************/
float CBot::yaw_to_player(int lookPlayerNum)
{
	// Bail out if it is an invalid player
	if (lookPlayerNum == -1)
		return 0;

	// Find the direction to the player
	CBaseEntity * pPlayerToFind = UTIL_PlayerByIndex(lookPlayerNum);
//	float yaw = YawToEntity(pPlayerToFind);

	if(pPlayerToFind == NULL || pPlayerToFind->pev->deadflag != DEAD_NO || pPlayerToFind->pev->effects & EF_NODRAW)
		return 0;


	if (gpGlobals->time >= nextAimErrorChange)
	{
		int highErrorRange = (100 - aimSkill) / 2;
		int lowErrorRange = -highErrorRange;

		xError = RANDOM_FLOAT(lowErrorRange, highErrorRange);
		yError = RANDOM_FLOAT(lowErrorRange, highErrorRange);
		zError = RANDOM_FLOAT(lowErrorRange, highErrorRange);

		nextAimErrorChange = gpGlobals->time + 0.5;
	}

	float yaw = YawToLocation( Vector(pPlayerToFind->pev->origin.x + xError,
									  pPlayerToFind->pev->origin.y + yError,
									  pPlayerToFind->pev->origin.z + zError) );

	//sprintf(this->m_debugStr, "error-x-y-z %d-%d-%d", (int)xError, (int)yError, (int)zError);
	//SendExternalDebug(playerNum, m_debugStr);

	// Introduce error into the sensor reading for lower-skilled bots
	/*if (gpGlobals->time >= nextAimErrorChange)
	{
		percentTurnError = RANDOM_FLOAT(-(100 - (float)aimSkill)/100, (100 - (float)aimSkill)/100);
		nextAimErrorChange = gpGlobals->time + 1.0;

		sprintf(this->m_debugStr, "percent-turn-error %f", percentTurnError);
		SendExternalDebug(playerNum, m_debugStr);
	} */
	if (fabs(yaw) < 15)
	{
		float maxError = CalculateSensorError(lookPlayerNum);
		//percentTurnError = RANDOM_FLOAT(-((100 - (float)aimSkill)/100) * maxError,
		//								((100 - (float)aimSkill)/100) * maxError);
		percentTurnError = RANDOM_FLOAT(-1.0,
										((100 - (float)aimSkill)/100) * maxError);
	}
	else
		percentTurnError = 0;

	// sprintf(this->m_debugStr, "percent-turn-error %f", percentTurnError);
	// SendExternalDebug(playerNum, m_debugStr);

//	float error = yaw * percentTurnError;
//	yaw += error;

	return yaw;
}

/*********************************************
/ PITCH TO PLAYER
/ Calculates the pitch (angle) between where the bot
/ is looking and the player of client index lookPlayerNum.
/
/ Return values are -180 <= yaw <= 180. If the player
/ is higher, the return value will be positive. Below,
/ the return will be negative.
*********************************************/
float CBot::pitch_to_player(int lookPlayerNum)
{
	// Bail out if it is an invalid player
	if (lookPlayerNum == -1)
		return 0;

	// Get the pitch of the current viewing angle
	float viewPitch = -pev->v_angle.x;

	// Find the player
	CBasePlayer * pPlayerToFind = (CBasePlayer *)UTIL_PlayerByIndex(lookPlayerNum);
	if (pPlayerToFind == NULL)
		return 0;

	// Get the pitch of the vector from the bot to the player's body
	Vector toplayer = pPlayerToFind->pev->origin - GetGunPosition();
	float rawPitchToPlayer = UTIL_VecToAngles(toplayer).x;

	// The calculated pitch will be the difference
	float pitch = rawPitchToPlayer - viewPitch;

	// Tweak the numbers a bit so they make more sense to the user
	if (pitch > 180)
		pitch -= 360;
	else if (pitch < -180)
		pitch += 360;

	// Return the difference (pitch from where the bot is looking)
	return pitch;
}

/*********************************************
/ DISTANCE TO PLAYER
/ Calculates the distance between the bot and a
/ player of client index lookPlayerNum.
/
/ Return value rounds the distance to an integer
/ because units of less than one are pretty insignificant
/ (especially compared to the performance gains of not
/ working with a float).
*********************************************/
int CBot::distance_to_player(int lookPlayerNum)
{
	// Bail out if it is an invalid player
	if (lookPlayerNum == -1)
		return 0;

	// Find the player
	CBaseEntity * pPlayerToFind = UTIL_PlayerByIndex(lookPlayerNum);
	if (pPlayerToFind == NULL)
		return 0;

	// Calculate the distance to the player
	Vector toplayer = pPlayerToFind->pev->origin - pev->origin;
	return (int)toplayer.Length();
}

/*********************************************
/ GET VISION DISTANCE
/ Calculates the distance along a beam until an obstacle is hit.
/ The beam that is tested is checkAngle degrees from the bot's
/ point of view.
*********************************************/
int CBot::get_vision_distance(float lookAngle, int checkDistance)
{
	return get_vision_info(lookAngle, checkDistance, NULL).vdistance;
}

/*********************************************
/ GET VISION DISTANCE
/ Calculates the distance along a beam until an obstacle is hit.
/ The beam that is tested is checkAngle degrees from the bot's
/ point of view.
*********************************************/
int CBot::get_vision_distance(float lookAngle, float pitch, int checkDistance)
{
	//if(!m_pBeam)
	//	m_pBeam = InitNewDistanceBeam(0,255,0);
	return get_vision_info(lookAngle, pitch, checkDistance, NULL).vdistance;
}


/*********************************************
/ GET VISION SWEEP DISTANCE
/ Calculates the distance the closest object hit 
/ in a sweep of tracelines (within the given sweep
/ range at the given intervals).
*********************************************/
int CBot::get_vision_sweep_distance(float lookAngle, int checkDistance, int sweep, int interval)
{
	return get_sweep_info(lookAngle, checkDistance, sweep, interval).vdistance;
}

/*********************************************
/ NEAREST VISIBLE ENEMY NUMBER
/ Returns the client index of the nearest visible enemy.
/ Returns -1 if no enemy is visible.
*********************************************/
int CBot::nearest_vis_enemy_num()
{
	int nearestEnemy = -1;

	// Check each player
	for (int currentPlayer = 1; currentPlayer <= gpGlobals->maxClients; currentPlayer++)
	{
		CBaseEntity *pPlayer = UTIL_PlayerByIndex( currentPlayer );

		// Skip invalid players and skip self (i.e. this bot)
		if ((!pPlayer) || (pPlayer == this))
			continue;
	
		// Is team play enabled?
		if (g_pGameRules->IsTeamplay())
		{
			// Don't target your teammates if team names match
			if (UTIL_TeamsMatch(g_pGameRules->GetTeamID(this), g_pGameRules->GetTeamID(pPlayer)))
				continue;
		}

		// Is the player visible?
		if (is_player_visible(currentPlayer))
		{
			// Is this the first visible enemy?
			if (nearestEnemy == -1)
				nearestEnemy = currentPlayer;
			// Is this player closer than the old nearest enemy?
			else if (distance_to_player(currentPlayer) < distance_to_player(nearestEnemy))
				nearestEnemy = currentPlayer;
		}
	}
	return nearestEnemy;
}

/*********************************************
/ GET VISION INFO
/ Calculates the distance along a beam until an obstacle is hit.
/ Also finds what obstacle was hit. The beam that is tested is
/ checkAngle degrees from the bot's point of view.
*********************************************/
VisionResult CBot::get_vision_info(float checkAngle, int checkDistance, CBeam * pBeam)
{
	TraceResult tr;
	VisionResult result;

	// Construct a vector from the origin of length checkDistance
	Vector visionVec (checkDistance, 0, 0);
	
	// Rotate the vision vector by checkAngle degrees from the player's view
	float rotateAngle = pev->v_angle.y + checkAngle;
	visionVec = BotUTIL_RotateVectorZ(visionVec, rotateAngle);

	// Trace a ray for the vision along this vector
	UTIL_TraceLine (pev->origin + pev->view_ofs,
					pev->origin + pev->view_ofs + visionVec,
					dont_ignore_monsters,
					pev->pContainingEntity,
					&tr);

	// Set up the beam for the next redraw (debugging purposes only)
	if (pBeam && m_bDebugOn)
		pBeam->SetStartPos( tr.vecEndPos );

	// Find out what (if any) type of obstacle was encountered
	Vector toObstacle = tr.vecEndPos - (pev->origin + pev->view_ofs);
	result.vdistance = (int)toObstacle.Length();

	// ** COMMENTED OUT BECAUSE IT IS NOT USED AND IS A SYSTEM HIT **
	// ** maybe put this back in later **
	// strcpy(result.ent_class, STRING(tr.pHit->v.classname));
	
	// Return the distance from the player to an obstacle (max is checkDistance)
	return result;
}
/*********************************************
/ GET VISION INFO
/ Calculates the distance along a beam until an obstacle is hit. 
/ pitch added to this version
/ Also finds what obstacle was hit. The beam that is tested is
/ checkAngle degrees from the bot's point of view.
*********************************************/
VisionResult CBot::get_vision_info(float checkAngle, float checkPitch, int checkDistance, CBeam * pBeam)
{
	TraceResult tr;
	VisionResult result;
	
	// Construct a vector from the origin of length checkDistance
	Vector visionVec (checkDistance * cos(degToRad(checkPitch)) , 0 , checkDistance*sin(degToRad(checkPitch)));
	
	// Rotate the vision vector by checkAngle degrees from the player's view
	float rotateAngle = pev->v_angle.y + checkAngle;
	visionVec = BotUTIL_RotateVectorZ(visionVec, rotateAngle);
	//rotateAngle = pev->v_angle.z + checkPitch;
	//visionVec = BotUTIL_RotateVectorY(visionVec, rotateAngle);	

	// Trace a ray for the vision along this vector
	UTIL_TraceLine (pev->origin + pev->view_ofs,
					pev->origin + pev->view_ofs + visionVec,
					dont_ignore_monsters,
					pev->pContainingEntity,
					&tr);

	// Set up the beam for the next redraw (debugging purposes only)
	if (pBeam && m_bDebugOn)
		pBeam->SetStartPos( tr.vecEndPos );

	// Find out what (if any) type of obstacle was encountered
	Vector toObstacle = tr.vecEndPos - (pev->origin + pev->view_ofs);
	result.vdistance = (int)toObstacle.Length();

	// ** COMMENTED OUT BECAUSE IT IS NOT USED AND IS A SYSTEM HIT **
	// ** maybe put this back in later **
	// strcpy(result.ent_class, STRING(tr.pHit->v.classname));
	
	// Return the distance from the player to an obstacle (max is checkDistance)
	return result;
}

/*********************************************
/ YAW TO ENTITY
/ Calculates the yaw (angle) between where the bot
/ is looking and an entity.
/
/ Return values are -180 <= yaw <= 180. If the entity
/ is to the right of center, the return value will be
/ negative. To the left, the return will be positive.
*********************************************/
float CBot::YawToEntity(CBaseEntity * pEnt)
{
	if(!pEnt)
		return 0;
	if(!pEnt->pev)
		return 0;
	// Find the direction that the player is currently facing
	float facingAngle = pev->v_angle.y;

	Vector vToEntity = pEnt->pev->origin - pev->origin;
	float angleToEntity = UTIL_VecToAngles(vToEntity).y;

	// Calculate the yaw between the two angles
	float yaw = facingAngle - angleToEntity;
	if (yaw < -180)
		yaw += 360;
	if (yaw > 180)
		yaw -=360;
	return yaw;
}

/*********************************************
/ YAW TO LOCATION
/ Calculates the yaw (angle) between where the bot
/ is looking and a location.
/
/ Return values are -180 <= yaw <= 180. If the location
/ is to the right of center, the return value will be
/ negative. To the left, the return will be positive.
*********************************************/
float CBot::YawToLocation(Vector location)
{
	// Find the direction that the player is currently facing
	float facingAngle = pev->v_angle.y;

	Vector vToEntity = location - pev->origin;
	float angleToEntity = UTIL_VecToAngles(vToEntity).y;

	// Calculate the yaw between the two angles
	float yaw = facingAngle - angleToEntity;
	if (yaw < -180)
		yaw += 360;
	if (yaw > 180)
		yaw -=360;
	return yaw;
}

/*********************************************
/ DISTANCE TO ENTITY
/ Calculates the distance between the bot and an entity.
*********************************************/
int CBot::DistanceToEntity(CBaseEntity * pEnt)
{
	if(!pEnt)
		return 0;
	if(!pEnt->pev)
		return 0;

	Vector vToEntity = pEnt->pev->origin - pev->origin;
	return (int)vToEntity.Length();
}

/*********************************************
/ DISTANCE TO LOCATION
/ Calculates the distance between the bot and a location.
*********************************************/
int CBot::DistanceToLocation(Vector location)
{
	Vector vToEntity = location - pev->origin;
	return (int)vToEntity.Length();
}

/*********************************************
/ PITCH TO ENTITY
/ Calculates the pitch between the bot and an entity.
*********************************************/
float CBot::PitchToEntity(CBaseEntity * pEnt)
{
	if(!pEnt)
		return 0;
	if(!pEnt->pev)
		return 0;

	// Get the pitch of the current viewing angle
	float viewPitch = -pev->v_angle.x;

	// Get the pitch of the vector from the bot to the entity's origin
	Vector toplayer = pEnt->pev->origin - GetGunPosition();
	float rawPitchToPlayer = UTIL_VecToAngles(toplayer).x;

	// The calculated pitch will be the difference
	float pitch = rawPitchToPlayer - viewPitch;

	// Tweak the numbers a bit so they make more sense to the user
	if (pitch > 180)
		pitch -= 360;
	else if (pitch < -180)
		pitch += 360;

	// Return the difference (pitch from where the bot is looking)
	return pitch;
}

/*********************************************
/ PITCH TO LOCATION
/ Calculates the pitch between the bot and a location.
*********************************************/
float CBot::PitchToLocation(Vector location)
{
	// Get the pitch of the current viewing angle
	float viewPitch = -pev->v_angle.x;

	// Get the pitch of the vector from the bot to the entity's origin
	Vector toplayer = location - GetGunPosition();
	float rawPitchToPlayer = UTIL_VecToAngles(toplayer).x;

	// The calculated pitch will be the difference
	float pitch = rawPitchToPlayer - viewPitch;

	// Tweak the numbers a bit so they make more sense to the user
	if (pitch > 180)
		pitch -= 360;
	else if (pitch < -180)
		pitch += 360;

	// Return the difference (pitch from where the bot is looking)
	return pitch;
}

/*****************************************
/IS MOVING
/to sense weather a bot has moved, only a rotation is 
/counted as not moved
******************************************/
bool CBot::is_moving()
{
	//check to se if the location information from this
	//time is the same as the previous location info	
	Vector moved = pev->origin - v_prev_origin;
	if (moved.Length() < 1) {
		return FALSE;
	}
	return TRUE;
}


/*********************************************
/ YAW TO WAYPOINT
/ Calculates the yaw (angle) between where the bot
/ is looking and a the waypoint with index of waypointindex
/
/ Return values are -180 <= yaw <= 180. If the player
/ is to the right of center, the return value will be
/ negative. To the left, the return will be positive.
*********************************************/
float CBot::yaw_to_waypoint(int waypoint_index)
{
	// Bail out if it is an invalid player
	if (waypoint_index == -1 || waypoint_index >= m_nWaypoints)
		return 0;
 
	// Find the direction to the waypoint
	CBaseEntity * pWaypoint = waypointBeams[waypoints[waypoint_index].beamLocation].pDisplayBeam;
	float yaw = YawToEntity(pWaypoint);
	return yaw;

}


/*********************************************
/ DISTANCE TO WAYPOINT
/ Calculates the distance to where the bot
/ is and the waypoint with index of waypointindex
*********************************************/
int CBot::distance_to_waypoint(int waypoint_index)
{
	// Bail out if it is an invalid waypoint
	if (waypoint_index == -1 || waypoint_index >= m_nWaypoints)
		return 0;
	// Find the player
	CBaseEntity * pWaypoint = waypointBeams[waypoints[waypoint_index].beamLocation].pDisplayBeam;
	if (pWaypoint == NULL)
		return 0;

	// Calculate the distance to the player
	Vector toWP = pWaypoint->pev->origin - pev->origin;
	return (int)toWP.Length();
}
/*********************************************
/ PITCH TO WAYPOINT
/ Returns the index number of the closest waypoint.
/ Returns -1 if no waypoints exist.
*********************************************/
float CBot::pitch_to_waypoint(int index)
{
	if (index < 0)
		return 0;

	if (index < m_nWaypoints && !waypoints[index].removed){
		CBaseEntity * Pwaypoint = waypointBeams[waypoints[index].beamLocation].pDisplayBeam;
		return PitchToEntity(Pwaypoint);
	}	
	return 0;
}
/*********************************************
/ NEAREST WAYPOINT NUMBER
/ Returns the index number of the closest waypoint.
/ Returns -1 if no waypoints exist.
*********************************************/
int CBot::nearest_waypoint_num()
{
	int nearestWaypoint = -1;
	// Check each waypoint
	for (int currentWaypoint = 0; currentWaypoint < m_nWaypoints; currentWaypoint++)
	{
		if(waypoints[currentWaypoint].removed == FALSE){
			if (nearestWaypoint == -1)//{
				nearestWaypoint = currentWaypoint;
		}
		// Is this waypoint closer than the old nearest waypoint?
		if (distance_to_waypoint(currentWaypoint) < distance_to_waypoint(nearestWaypoint))
			nearestWaypoint = currentWaypoint;
		
	}
	return nearestWaypoint;
}

/*********************************************
/ YAW TO INFOPOINT
/ Calculates the yaw (angle) between where the bot
/ is looking and a the infopoint with index of infopointindex
/
/ Return values are -180 <= yaw <= 180. If the player
/ is to the right of center, the return value will be
/ negative. To the left, the return will be positive.
*********************************************/
float CBot::yaw_to_infopoint(int infopoint_index, int type)
{
	if (infopoint_index == -1 || infopoint_index >= m_nInfopoints)
		return 0;
	//Find the direction to the infopoint
	CBaseEntity * Pinfopoint = InfopointBeams[(Infopoints[infopoint_index].beamLocation)].pDisplayBeam;
	if (Pinfopoint == NULL)
		return -1;
	float yaw = YawToEntity(Pinfopoint);
	return yaw;
}

/*********************************************
/ DISTANCE TO INFOPOINT
/ Calculates the distance to where the bot
/ is and the infopoint with index of infopointindex
*********************************************/
int CBot::distance_to_infopoint(int infopoint_index, int type)
{
	if (infopoint_index == -1 || infopoint_index >= m_nInfopoints)
		return -1;

	if(Infopoints[infopoint_index].waypoint.type != type)
		return -1;

	CBaseEntity * Pinfopoint = (InfopointBeams[(Infopoints[infopoint_index].beamLocation)]).pDisplayBeam;
		//Find the direction to the infopoint
	//bail if the pointer is invalid
	if (Pinfopoint == NULL)
		return -1;
	
	Vector toIP = Pinfopoint->pev->origin - pev->origin;
	return (int)toIP.Length();
}	

/*********************************************
/ NEAREST INFOPOINT NUMBER
/ Returns the index number of the closest infopoint.
/ Returns -1 if no waypoints exist.
*********************************************/
int CBot::nearest_infopoint_num(int type)
{
	int nearestInfopoint = -1;
	// Check each waypoint
	
	for (int currentInfopoint = 0; currentInfopoint < m_nInfopoints; currentInfopoint++)
	{
		if(Infopoints[currentInfopoint].removed == FALSE)
			if(Infopoints[currentInfopoint].waypoint.type == type)
			{
				if(nearestInfopoint == -1)
					nearestInfopoint = currentInfopoint;
				// Is this waypoint closer than the old nearest waypoint?
				else if(distance_to_infopoint(currentInfopoint, type) < distance_to_infopoint(nearestInfopoint,type))
					nearestInfopoint = currentInfopoint;
			}
	}
	return nearestInfopoint;
}

/*********************************************
/ NEXT INFOPOINT TO EXPIRE NUMBER
/ Returns the index number of the next infopoint to expire
/ Returns -1 if no waypoints exist.
*********************************************/
int CBot::next_ip_to_expire_num(int type)
{
	if (m_nInfopoints == 0)
		return -1;
	int nextInfopoint = -1;
	// Check each infopoint
	for (int currentInfopoint = 0; currentInfopoint < m_nInfopoints; currentInfopoint++)
	{
		if(Infopoints[currentInfopoint].removed == FALSE){
			if(Infopoints[currentInfopoint].waypoint.type == type)
				if(nextInfopoint == -1)
					nextInfopoint = currentInfopoint;
				// Is this infopoint closer than the old next infopoint?
				else if(Infopoints[currentInfopoint].startTime + Infopoints[currentInfopoint].lifeTime < Infopoints[nextInfopoint].startTime + Infopoints[nextInfopoint].lifeTime)
					nextInfopoint = currentInfopoint;
		}
	}
	return nextInfopoint;
}

/*****************************************
/PITCH TO INFOPOINT
/returns pitch to the infopoint 
******************************************/
float CBot::pitch_to_infopoint(int infopointType, int index)
{
	if (index < 0 || index > m_nInfopoints)
		return -1;
	
	if (index < m_nInfopoints && !Infopoints[index].removed)
	{
		if(Infopoints[index].waypoint.type == infopointType)
		{
			CBaseEntity * Pinfopoint = InfopointBeams[(Infopoints[index]).beamLocation].pDisplayBeam;
			return PitchToEntity(Pinfopoint);
		}
	}
	return -1;
}

/*****************************************
/AMMO IN CLIP
/To sense when a gun is empty, for the porpouse
/of reloading 
******************************************/
int CBot::ammo_in_clip()
{
	//sometimes a weapon can take sriaght form a source other than the clip so check that if the clipp is empty
	if( ((CBasePlayerWeapon *)(this->m_pActiveItem))->m_iClip == -1){
		return this->m_rgAmmo[((CBasePlayerWeapon *)(this->m_pActiveItem))->m_iPrimaryAmmoType];
	}
	return ((CBasePlayerWeapon *)(this->m_pActiveItem))->m_iClip;
}


/*****************************************
/PLAYER_NUM
/to give the bots knowledge of who they are
******************************************/
int CBot::player_num()
{
	return playerNum;
}


/*****************************************
/ NEAREST_TEAMMATE
/ To sense when a gun is empty, for the porpouse
/ of reloading 
******************************************/
int CBot::nearest_vis_teammate_num()
{
	int nearestEnemy = -1;

	// Check each player
	for (int currentPlayer = 1; currentPlayer <= gpGlobals->maxClients; currentPlayer++)
	{
		CBaseEntity *pPlayer = UTIL_PlayerByIndex( currentPlayer );

		// Skip invalid players and skip self (i.e. this bot)
		if ((!pPlayer) || (pPlayer == this))
			continue;
		// Is team play enabled? otherwise you have no freinds
		if (g_pGameRules->IsTeamplay())
		{
			// target your teammates, if team names match then a teammate
			if (UTIL_TeamsMatch(g_pGameRules->GetTeamID(this), g_pGameRules->GetTeamID(pPlayer)))
			{
				// Is the player visible?
				if (is_player_visible(currentPlayer))
				{
					// Is this the first visible enemy?
					if (nearestEnemy == -1)
						nearestEnemy = currentPlayer;
					// Is this player closer than the old nearest enemy?
					else if (distance_to_player(currentPlayer) < distance_to_player(nearestEnemy))
						nearestEnemy = currentPlayer;
				}
			}
		
		}
	}
	return nearestEnemy;
}

/************************************************
/ YAW OF ENEMY SOUNDS
/ provides the yaw to the closests enemy sound to the bot
*************************************************/
float CBot::yaw_to_enemy_sound()
{
	Vector toSound;
	int nearestEnemy = -1;
	int distance_to_sound;
	float yaw = 0;
	CBasePlayer *pPlayer = NULL ;

	for(int currentPlayer = 1; currentPlayer <= gpGlobals->maxClients; currentPlayer++)
	{
		pPlayer = (CBasePlayer *)UTIL_PlayerByIndex( currentPlayer );
		if ((!pPlayer) || (pPlayer == this))
			continue;

		if(g_pGameRules->IsTeamplay())
			// Don't target your teammates if team names match
			if (UTIL_TeamsMatch(g_pGameRules->GetTeamID(this), g_pGameRules->GetTeamID(pPlayer)))
				continue;
		
		distance_to_sound = (int)distance_to_player(currentPlayer);
		toSound = pPlayer->pev->origin - pev->origin;
		// check to see if how far away to sound is, to see if it is fair that the bot would hear it
		if(distance_to_sound >  3 * pPlayer->soundVolume)
		{
			continue;	
		}
	
		if (nearestEnemy == -1)
			nearestEnemy = currentPlayer;
		// Is this player closer than the old nearest enemy?
		else if (distance_to_player(currentPlayer) < distance_to_player(nearestEnemy))
			nearestEnemy = currentPlayer;
	}

	if(nearestEnemy != -1)
		yaw = (float)yaw_to_player(nearestEnemy);
	
	return (float)yaw;
}

/************************************************
/ DISTANCE OF ENEMY SOUNDS
/ provides the yaw to the closests enemy sound to the bot
*************************************************/
int CBot::distance_to_enemy_sound()
{
	Vector toSound;
	int nearestEnemy = -1;
	int distance_to_sound;
	int distance = 0;
	CBasePlayer *pPlayer = NULL ;

	for(int currentPlayer = 1; currentPlayer <= gpGlobals->maxClients; currentPlayer++)
	{
		pPlayer = (CBasePlayer *)UTIL_PlayerByIndex( currentPlayer );
		if ((!pPlayer) || (pPlayer == this))
			continue;

		if(g_pGameRules->IsTeamplay())
			// Don't target your teammates if team names match
			if (UTIL_TeamsMatch(g_pGameRules->GetTeamID(this), g_pGameRules->GetTeamID(pPlayer)))
				continue;
		
		distance_to_sound = (int)distance_to_player(currentPlayer);
		toSound = pPlayer->pev->origin - pev->origin;
		// Check to see if how far away to sound is, to see if it is fair that the bot would hear it
		if(distance_to_sound >  3 * pPlayer->soundVolume)
		{
			continue;	
		}
	
		if (nearestEnemy == -1)
			nearestEnemy = currentPlayer;
		// Is this player closer than the old nearest enemy?
		else if (distance_to_player(currentPlayer) < distance_to_player(nearestEnemy))
			nearestEnemy = currentPlayer;
	}

	if(nearestEnemy != -1)
		distance = (int)distance_to_player(nearestEnemy);
	
	return distance;
}

/************************************************
/ CLOSEST SURROUNDING ITEM
/ Returns the index of the closes surrounding item
/ for the item inventory provided.
*************************************************/
int CBot::ClosestSurroundingItem(BotSurroundingItems * itemInventory)
{
	int closestItem = 0;

	if (!itemInventory)
		return -1;
	
	// Return a bogus index if there are no surrounding items of this type
	if (itemInventory->numItems == 0)
		return -1;
	
	else
	{
		// Check each item
		for (int i = 1; i < itemInventory->numItems; i++)
		{
			if (UTIL_IsValidEntity(itemInventory->pEdicts[i]))
			{
				if (!IsEntityVisible(itemInventory->items[closestItem]))
					closestItem = i;
				else if (DistanceToEntity(itemInventory->items[i]) < DistanceToEntity(itemInventory->items[closestItem]))
				{
					if (IsEntityVisible(itemInventory->items[i]))
						closestItem = i;
				}
			}
		}
	}

	if (UTIL_IsValidEntity(itemInventory->pEdicts[closestItem]))
	{
		if (!IsEntityVisible(itemInventory->items[closestItem]))
			return -1;
		else 
			return closestItem;
	}
	else
		return -1;
	
}

/************************************************
/ CLOSEST SURROUNDING ITEM
/ Returns the index of the closes surrounding item
/ for the item inventory provided.
*************************************************/
int CBot::ClosestSurroundingUnstableItem(BotSurroundingItems * itemInventory)
{
	int closestItem = 0;

	if (!itemInventory)
		return -1;
	
	// Return a bogus index if there are no surrounding items of this type
	if (itemInventory->numItems == 0)
		return -1;
	else
	{
		// Check each item
		for (int i = 1; i < itemInventory->numItems; i++)
		{
			if (UTIL_IsValidEntity(itemInventory->pEdicts[i]))
			{
				if (!IsLocationVisible(itemInventory->origins[closestItem]))
					closestItem = i;
				else if (DistanceToLocation(itemInventory->origins[i]) < DistanceToLocation(itemInventory->origins[closestItem]))
				{
					if (IsLocationVisible(itemInventory->origins[i]))
						closestItem = i;
				}
			}
		}
	}

	if (UTIL_IsValidEntity(itemInventory->pEdicts[closestItem]))
	{
		if (!IsLocationVisible(itemInventory->origins[closestItem]))
			return -1;
		else 
			return closestItem;
	}
	else
		return -1;
}
/************************************************
/ CLOSEST SURROUNDING LADDER ITEM
/ Returns the index of the closes surrounding item
/ for the item inventory provided.
*************************************************/
int CBot::ClosestSurroundingLadderItem(BotSurroundingItems * itemInventory)
{
	int closestItem = 0;
	
	if (!itemInventory)
		return -1;
	
	// Return a bogus index if there are no surrounding items of this type
	if (itemInventory->numItems == 0)
		return -1;
	else
	{
		// Check each item
		for (int i = 1; i < itemInventory->numItems; i++)
		{
			if (UTIL_IsValidEntity(itemInventory->pEdicts[i]))
			{
				if (!FVisible(surroundings.ladderItems.items[closestItem]))
					closestItem = i;
				
				if (DistanceToEntity(itemInventory->items[i]) < DistanceToEntity(itemInventory->items[closestItem]))
				{
					if (FVisible(surroundings.ladderItems.items[closestItem]))
						closestItem = i;
				}
			}
		}
	}
	
	if (UTIL_IsValidEntity(itemInventory->pEdicts[closestItem]))
	{
		
		//if (!FVisible(surroundings.ladderItems.items[closestItem]))
	//		return -1;
	//	else
			return closestItem;
	}
	else
		return -1;
}

/************************************************
/ NEAREST ITEM
/ Returns the yaw to the nearest piece of ammo.
*************************************************/
int CBot::nearest_item(int itemType)
{
	switch (itemType)
	{
	case ItemTypeWeapon:
		return ClosestSurroundingItem(&surroundings.weaponItems);		
	case ItemTypeAmmo:
		return ClosestSurroundingItem(&surroundings.ammoItems);
	case ItemTypeWeaponbox:
		return ClosestSurroundingUnstableItem(&surroundings.weaponBoxItems);
	case ItemTypeLadder:
		return ClosestSurroundingLadderItem(&surroundings.ladderItems);
	case ItemTypeHealthKit:
		return ClosestSurroundingItem(&surroundings.healthKitItems);
	case ItemTypeHealthCharger:
		return ClosestSurroundingItem(&surroundings.healthChargerItems);
	case ItemTypeSuitCharger:
		return ClosestSurroundingItem(&surroundings.suitChargerItems);
	}
	return -1;
}

/************************************************
/ YAW TO ITEM
/ Returns the yaw to the nearest piece of ammo.
*************************************************/
float CBot::yaw_to_item(int itemType, int index)
{
	if (index < 0)
		return FALSE;

	switch (itemType)
	{
	case ItemTypeWeapon:
		if (index < surroundings.weaponItems.numItems)
			return YawToEntity(surroundings.weaponItems.items[index]);
		else
			return 0;
		
	case ItemTypeAmmo:
		if (index < surroundings.ammoItems.numItems)
			return YawToEntity(surroundings.ammoItems.items[index]);
		else
			return 0;

	case ItemTypeWeaponbox:
		if (index < surroundings.weaponBoxItems.numItems)
			// return YawToEntity(surroundings.weaponBoxItems.items[index]);
			return YawToLocation(surroundings.weaponBoxItems.origins[index]);
		else
			return 0;

	case ItemTypeLadder:
		if (index < surroundings.ladderItems.numItems)
			return YawToLocation(surroundings.ladderItems.origins[index]);
		else
			return 0;
		
	case ItemTypeHealthKit:
		if (index < surroundings.healthKitItems.numItems)
			return YawToLocation(surroundings.healthKitItems.origins[index]);
		else
			return 0;

	case ItemTypeHealthCharger:
		if (index < surroundings.healthChargerItems.numItems)
			return YawToEntity(surroundings.healthChargerItems.items[index]);
		else
			return 0;

	case ItemTypeSuitCharger:
		if (index < surroundings.suitChargerItems.numItems)
			return YawToEntity(surroundings.suitChargerItems.items[index]);
		else
			return 0;
	}

	return 0;
}

/************************************************
/ DISTANCE TO ITEM
/ Returns the distance to the nearest piece of ammo.
*************************************************/
int CBot::distance_to_item(int itemType, int index)
{
	if (index < 0)
		return FALSE;

	switch (itemType)
	{
	case ItemTypeWeapon:
		if (index < surroundings.weaponItems.numItems)
			return DistanceToEntity(surroundings.weaponItems.items[index]);
		else
			return 0;
		
	case ItemTypeAmmo:
		if (index < surroundings.ammoItems.numItems)
			return DistanceToEntity(surroundings.ammoItems.items[index]);
		else
			return 0;

	case ItemTypeWeaponbox:
		if (index < surroundings.weaponBoxItems.numItems)
			// return DistanceToEntity(surroundings.weaponBoxItems.items[index]);
			return DistanceToLocation(surroundings.weaponBoxItems.origins[index]);
		else
			return 0;

	case ItemTypeLadder:
		if (index < surroundings.ladderItems.numItems)
			return DistanceToLocation(surroundings.ladderItems.origins[index]);
		else
			return 0;

	case ItemTypeHealthKit:
		if (index < surroundings.healthKitItems.numItems)
			return DistanceToEntity(surroundings.healthKitItems.items[index]);
		else
			return 0;
	
	case ItemTypeHealthCharger:
		if (index < surroundings.healthChargerItems.numItems)
			return DistanceToEntity(surroundings.healthChargerItems.items[index]);
		else
			return 0;

	case ItemTypeSuitCharger:
		if (index < surroundings.suitChargerItems.numItems)
			return DistanceToLocation(surroundings.suitChargerItems.origins[index]);
		else
			return 0;
	}

	return 0;
}

/************************************************
/ PITCH TO ITEM
/ Returns the pitch to the nearest piece of ammo.
*************************************************/
float CBot::pitch_to_item(int itemType, int index)
{
	if (index < 0)
		return FALSE;

	switch (itemType)
	{
	case ItemTypeWeapon:
		if (index < surroundings.weaponItems.numItems)
			return PitchToEntity(surroundings.weaponItems.items[index]);
		else
			return 0;

	case ItemTypeAmmo:
		if (index < surroundings.ammoItems.numItems)
			return PitchToEntity(surroundings.ammoItems.items[index]);
		else
			return 0;
	case ItemTypeWeaponbox:
		if (index < surroundings.weaponBoxItems.numItems)
			// return PitchToEntity(surroundings.weaponBoxItems.items[index]);
			return PitchToLocation(surroundings.weaponBoxItems.origins[index]);
		else
			return 0;

	case ItemTypeLadder:
		if (index < surroundings.ladderItems.numItems)
			return PitchToEntity(surroundings.ladderItems.items[index]);
		else
			return 0;

	case ItemTypeHealthKit:
		if (index < surroundings.healthKitItems.numItems)
			return PitchToLocation(surroundings.healthKitItems.origins[index]);
		else
			return 0;

	case ItemTypeHealthCharger:
		if (index < surroundings.healthChargerItems.numItems)
			return PitchToEntity(surroundings.healthChargerItems.items[index]);
		else
			return 0;

	case ItemTypeSuitCharger:
		if (index < surroundings.suitChargerItems.numItems)
			return PitchToEntity(surroundings.suitChargerItems.items[index]);
		else
			return 0;

	default:
		return 0;
	}
}

/************************************************
/ AMMO CHECK
/ Info necessary for checking whether a bot can
/ pick up ammo. Code by Botman.
*************************************************/
ammo_check_t ammo_check[] = {
   {"ammo_glockclip", "9mm", _9MM_MAX_CARRY},
   {"ammo_9mmclip", "9mm", _9MM_MAX_CARRY},
   {"ammo_9mmAR", "9mm", _9MM_MAX_CARRY},
   {"ammo_9mmbox", "9mm", _9MM_MAX_CARRY},
   {"ammo_mp5clip", "9mm", _9MM_MAX_CARRY},
   {"ammo_chainboxclip", "9mm", _9MM_MAX_CARRY},
   {"ammo_mp5grenades", "ARgrenades", M203_GRENADE_MAX_CARRY},
   {"ammo_ARgrenades", "ARgrenades", M203_GRENADE_MAX_CARRY},
   {"ammo_buckshot", "buckshot", BUCKSHOT_MAX_CARRY},
   {"ammo_crossbow", "bolts", BOLT_MAX_CARRY},
   {"ammo_357", "357", _357_MAX_CARRY},
   {"ammo_rpgclip", "rockets", ROCKET_MAX_CARRY},
   {"ammo_egonclip", "uranium", URANIUM_MAX_CARRY},
   {"ammo_gaussclip", "uranium", URANIUM_MAX_CARRY},
   {"", 0, 0}};

/************************************************
/ CAN GET ITEM
/ Returns whether the bot can actually obtain an item
/ (e.g. dosen't already have it). The ammo-checking
/ code is based on code from Botman.
*************************************************/
bool CBot::can_get_item(int itemType, int index)
{
	if (index < 0)
		return FALSE;

	// Get a pointer to the item
	switch (itemType)
	{
	case ItemTypeWeapon:
		if (index < surroundings.weaponItems.numItems)
		{
			/***************
			CBasePlayerWeapon * pWeapon = (CBasePlayerWeapon *)surroundings.weaponItems.items[index];
			if (UTIL_IsValidEntity(surroundings.weaponItems.pEdicts[index]))
			{
				if ( pWeapon->m_iDefaultAmmo == 17 || pWeapon->m_iDefaultAmmo == 0) // hack to fix odd crash (does it work?)
					return TRUE;
				else
					return g_pGameRules->CanHavePlayerItem( this, static_cast<CBasePlayerItem *>(surroundings.weaponItems.items[index]) );
			}
			else 
				return FALSE;
			****************/
			if (UTIL_IsValidEntity(surroundings.weaponItems.pEdicts[index]))
			{
				CBasePlayerItem * pWeapon = (CBasePlayerItem *)surroundings.weaponItems.items[index];
				if (get_possesses_gun( getWeaponIndex(pWeapon) ))
					return TRUE;
				else
					return FALSE;
			}
			else
				return FALSE;

			break;
		}
		else
			return FALSE;

	case ItemTypeAmmo:
		if (index < surroundings.ammoItems.numItems)
		{
			CBasePlayerAmmo * pAmmo = (CBasePlayerAmmo *)surroundings.ammoItems.items[index];
			int i = 0;
			while (ammo_check[i].ammo_name[0])
			{
				if (strcmp(ammo_check[i].ammo_name, STRING(pAmmo->pev->classname)) == 0)
  					// See if the bot can pick up this ammo
					if (UTIL_IsValidEntity(surroundings.ammoItems.pEdicts[index]))
						return (bool)(g_pGameRules->CanHaveAmmo( this, ammo_check[i].weapon_name, ammo_check[i].max_carry));
					else
						return FALSE;
				i++;
			}
		}
		else
			return FALSE;

	case ItemTypeWeaponbox:
		if (index < surroundings.weaponBoxItems.numItems)
			return TRUE;
		else
			return FALSE;

	case ItemTypeHealthKit:
		if (pev->health < 100)
			return TRUE;
		else
			return FALSE;

	case ItemTypeHealthCharger:
		if (index < surroundings.healthChargerItems.numItems)
		{
			CBaseEntity * pHealthCharger = (CBaseEntity *)surroundings.healthChargerItems.items[index];
			if (pHealthCharger->pev->frame == 0 && pev->health < 100)
				return TRUE;
			else
				return FALSE;
		}

	case ItemTypeSuitCharger:
		if (index < surroundings.suitChargerItems.numItems)
		{
			CBaseEntity * pSuitCharger = (CBaseEntity *)surroundings.suitChargerItems.items[index];
			if (pSuitCharger->pev->frame == 0 && pev->health < 100)
				return TRUE;
			else
				return FALSE;
		}

	default:
		return FALSE;
	}
}

/************************************************
/ GET X
/ a sensor for position of the bot in the level
/ potenail problems are that the map origin can be 
/ anywhere in the levelso valuse can be negative
*************************************************/
int CBot::get_x()
{	
	return(pev->origin.x);
}
/************************************************
/ GET Y
/ a sensor for position of the bot in the level
/ potenail problems are that the map origin can be 
/ anywhere in the levelso valuse can be negative
*************************************************/
int CBot::get_y()
{	
	return(pev->origin.y);
}
/************************************************
/ GET Z
/ a sensor for position of the bot in the level
/ potenail problems are that the map origin can be 
/ anywhere in the levelso valuse can be negative
*************************************************/
int CBot::get_z()
{	
	return(pev->origin.z);
}

/************************************************
/ IS FRIEND
/ check a player index to see if he is on the same
/ team, TRUE he is and FALSE he ain't
*************************************************/
bool CBot::is_friend(int player_index){
	if(player_index < 1)
		return FALSE;
	CBaseEntity *pPlayer = UTIL_PlayerByIndex( player_index );

	// Skip invalid players and skip self (i.e. this bot)
	if ((!pPlayer) || (pPlayer == this))
		return FALSE;

	// Is team play enabled?
	if (g_pGameRules->IsTeamplay())
	{
		if (UTIL_TeamsMatch(g_pGameRules->GetTeamID(this), g_pGameRules->GetTeamID(pPlayer)))
			return TRUE;
	}
	return FALSE;
}
/************************************************
/ GET POSSESSES GUN
/ check to see if the bot has the gun in question
*************************************************/
bool CBot::get_possesses_gun(int gun_num)
{
	if(gun_num > 0 || gun_num < 16){
		if(pev->weapons & (1<<gun_num))
			return TRUE;
	}	
	return FALSE;
}
/************************************************
/ PRIMARY AMMO IN STOCK
/ check ammo int the players reserve
*************************************************/
int CBot::primary_ammo_in_stock(int gun_num)
{	
	switch(gun_num){
	case WEAPON_NONE:
		return 0;
	case WEAPON_CROWBAR:
		return 1;//this->m_rgAmmo[GetAmmoIndex("weapon_crowbar")];
		break;
	case WEAPON_GLOCK:
		return this->m_rgAmmo[GetAmmoIndex("9mm")];
		break;
	case WEAPON_SHOTGUN:
		return this->m_rgAmmo[GetAmmoIndex("buckshot")];	
		break;
	case WEAPON_GAUSS:
		return this->m_rgAmmo[GetAmmoIndex("uranium")];
		break;
	case WEAPON_EGON:
		return this->m_rgAmmo[GetAmmoIndex("uranium")];
		break;
	case WEAPON_MP5:
		return this->m_rgAmmo[GetAmmoIndex("9mm")];
		break;
	case WEAPON_RPG:
		return this->m_rgAmmo[GetAmmoIndex("rockets")];
		break;
	case WEAPON_PYTHON:
		return this->m_rgAmmo[GetAmmoIndex("357")];
		break;
	case WEAPON_HORNETGUN:
		return 8;//this->m_rgAmmo[GetAmmoIndex("weapon_hornetgun")];
		break;
	case WEAPON_CROSSBOW:
		return this->m_rgAmmo[GetAmmoIndex("bolts")];
		break;
	case WEAPON_SNARK:
		return this->m_rgAmmo[GetAmmoIndex("Snarks")];
		break;
	case WEAPON_HANDGRENADE:
		return this->m_rgAmmo[GetAmmoIndex("Hand Grenade")];
		break;
	case WEAPON_TRIPMINE:
		return this->m_rgAmmo[GetAmmoIndex("Trip Mine")];
		break;
	case WEAPON_SATCHEL:
		return this->m_rgAmmo[GetAmmoIndex("Satchel Charge")];
		break;
	default:
		return 0;	
	}
	return 0;
}	
/************************************************
/ SECONDARY AMMO IN STOCK
/ check ammo int the players reserve
*************************************************/
int CBot::secondary_ammo_in_stock(int gun_num)
{
	return this->m_rgAmmo[((CBasePlayerWeapon *)(this->m_pActiveItem))->m_iSecondaryAmmoType];
	
	switch(gun_num){
	case WEAPON_GLOCK:
		return this->m_rgAmmo[GetAmmoIndex("9mm")];
		break;
	case WEAPON_SHOTGUN:
		return this->m_rgAmmo[GetAmmoIndex("buckshot")];	
		break;
	case WEAPON_GAUSS:
		return this->m_rgAmmo[GetAmmoIndex("uranium")];
		break;
	case WEAPON_MP5:
		return this->m_rgAmmo[GetAmmoIndex("ARgrenades")];
		break;
	default:
		return 0;
	}
	return 0;
}

/************************************************
/ CURRENT WEAPON
/ Checks which weapon is currently active. Returns
/ an integer representing the weapon (see the #define's
/ in "bot.h" for definitions).
*************************************************/

/*
#define WEAPON_NONE				0
#define WEAPON_CROWBAR			1
#define	WEAPON_GLOCK			2
#define WEAPON_PYTHON			3
#define WEAPON_MP5				4
#define WEAPON_CHAINGUN			5
#define WEAPON_CROSSBOW			6
#define WEAPON_SHOTGUN			7
#define WEAPON_RPG				8
#define WEAPON_GAUSS			9
#define WEAPON_EGON				10
#define WEAPON_HORNETGUN		11
#define WEAPON_HANDGRENADE		12
#define WEAPON_TRIPMINE			13
#define	WEAPON_SATCHEL			14
#define	WEAPON_SNARK			15
*/

int getWeaponIndex(CBasePlayerItem * weapon)
{
	const char * currentWeapon;
	currentWeapon = weapon->pszName();

	if (currentWeapon == NULL)
		return WEAPON_NONE;

	if (strlen(currentWeapon) == 0)
		return WEAPON_NONE;

	// Return the appropriate value if the weapon is known
	if (FStrEq(currentWeapon, "weapon_crowbar"))
		return WEAPON_CROWBAR;
	else if (FStrEq(currentWeapon, "weapon_9mmhandgun"))
		return WEAPON_GLOCK;
	else if (FStrEq(currentWeapon, "weapon_shotgun"))
		return WEAPON_SHOTGUN;
	else if (FStrEq(currentWeapon, "weapon_gauss"))
		return WEAPON_GAUSS;
	else if (FStrEq(currentWeapon, "weapon_egon"))
		return WEAPON_EGON;
	else if (FStrEq(currentWeapon, "weapon_9mmAR"))
		return WEAPON_MP5;
	else if (FStrEq(currentWeapon, "weapon_rpg"))
		return WEAPON_RPG;
	else if (FStrEq(currentWeapon, "weapon_357"))
		return WEAPON_PYTHON;
	else if (FStrEq(currentWeapon, "weapon_hornetgun"))
		return WEAPON_HORNETGUN;
	else if (FStrEq(currentWeapon, "weapon_crossbow"))
		return WEAPON_CROSSBOW;
	else if (FStrEq(currentWeapon, "weapon_snark"))
		return WEAPON_SNARK;
	else if (FStrEq(currentWeapon, "weapon_handgrenade"))
		return WEAPON_HANDGRENADE;
	else if (FStrEq(currentWeapon, "weapon_tripmine"))
		return WEAPON_TRIPMINE;
	else if (FStrEq(currentWeapon, "weapon_satchel"))
		return WEAPON_SATCHEL;


	// Fail an assert to help us add more weapon support (debugging)
	// assert(FALSE);
	return WEAPON_NONE;
}

int CBot::get_current_weapon()
{
	return getWeaponIndex(m_pActiveItem);
/*	const char * currentWeapon;
	currentWeapon = m_pActiveItem->pszName();

	if (strlen(currentWeapon) == 0)
		return -1;

	// Return the appropriate value if the weapon is known
	if (FStrEq(currentWeapon, "weapon_crowbar"))
		return WEAPON_CROWBAR;
	else if (FStrEq(currentWeapon, "weapon_9mmhandgun"))
		return WEAPON_GLOCK;
	else if (FStrEq(currentWeapon, "weapon_shotgun"))
		return WEAPON_SHOTGUN;
	else if (FStrEq(currentWeapon, "weapon_gauss"))
		return WEAPON_GAUSS;
	else if (FStrEq(currentWeapon, "weapon_egon"))
		return WEAPON_EGON;
	else if (FStrEq(currentWeapon, "weapon_9mmAR"))
		return WEAPON_MP5;
	else if (FStrEq(currentWeapon, "weapon_rpg"))
		return WEAPON_RPG;
	else if (FStrEq(currentWeapon, "weapon_357"))
		return WEAPON_PYTHON;
	else if (FStrEq(currentWeapon, "weapon_hornetgun"))
		return WEAPON_HORNETGUN;
	else if (FStrEq(currentWeapon, "weapon_crossbow"))
		return WEAPON_CROSSBOW;
	else if (FStrEq(currentWeapon, "weapon_snark"))
		return WEAPON_SNARK;
	else if (FStrEq(currentWeapon, "weapon_handgrenade"))
		return WEAPON_GRENADE;
	else if (FStrEq(currentWeapon, "weapon_tripmine"))
		return WEAPON_TRIPMINE;
	else if (FStrEq(currentWeapon, "weapon_satchel"))
		return WEAPON_SATCHEL;


	// Fail an assert to help us add more weapon support (debugging)
	// assert(FALSE);
	return WEAPON_NONE;
	*/
}

/*********************************************
/ ON LADDER
/ Returns true if the player is on a ladder
*********************************************
no sensor code is nessisary sinvce the player class
already has a IsOnLadder() function that the wrapper will
call directly

bool CBot::is_on_ladder()
{
m_iClientHealth is inherited from CBasePlayer
return this->IsOnLadder();
}
*/

int CBot::comms_get_int(int player_id, int box){
	//get the whole row that is specified in box
	int* array = (int*)hivemind_get(type_integer, box);	
	
		if(player_id < 0 && player_id > MAX_NUM_PLAYERS)
			return 0;
		return (int)array[player_id];
}

float CBot::comms_get_float(int player_id, int box){
	//get the whole row that is specified in box	
	float* array = (float*)hivemind_get(type_float, box);	
	
	if(player_id < 0 && player_id > MAX_NUM_PLAYERS)
		return 0;

	return (float)array[player_id];
}

bool CBot::comms_get_bool(int player_id, int box){
	//get the whole row that is specified in box
	bool* array = (bool*)hivemind_get(type_bool, box);	

	if(player_id < 0 && player_id > MAX_NUM_PLAYERS)
		return 0;
	return (bool)array[player_id];
}


		

